@@ -183,7 +183,14 @@ struct StrpTimeFunction {
 		auto &func_expr = state.expr.Cast<BoundFunctionExpression>();
 		auto &info = func_expr.bind_info->Cast<StrpTimeBindData>();
 
-		if (args.data[1].GetVectorType() == VectorType::CONSTANT_VECTOR && ConstantVector::IsNull(args.data[1])) {
+		//	There is a bizarre situation where the format column is foldable but not constant
+		//	(i.e., the statistics tell us it has only one value)
+		//	We have to check whether that value is NULL
+		const auto count = args.size();
+		UnifiedVectorFormat format_unified;
+		args.data[1].ToUnifiedFormat(count, format_unified);
+
+		if (!format_unified.validity.RowIsValid(0)) {
 			result.SetVectorType(VectorType::CONSTANT_VECTOR);
 			ConstantVector::SetNull(result, true);
 			return;
