@@ -298,9 +298,9 @@ unique_ptr<LocalSinkState> PhysicalBatchInsert::GetLocalSinkState(ExecutionConte
 	return make_uniq<BatchInsertLocalState>(context.client, insert_types, bound_defaults);
 }
 
-void PhysicalBatchInsert::NextBatch(ExecutionContext &context, GlobalSinkState &state, LocalSinkState &lstate_p) const {
-	auto &gstate = state.Cast<BatchInsertGlobalState>();
-	auto &lstate = lstate_p.Cast<BatchInsertLocalState>();
+SinkNextBatchType PhysicalBatchInsert::NextBatch(ExecutionContext &context, OperatorSinkNextBatchInput &input) const {
+	auto &gstate = input.global_state.Cast<BatchInsertGlobalState>();
+	auto &lstate = input.local_state.Cast<BatchInsertLocalState>();
 
 	auto &table = gstate.table;
 	auto batch_index = lstate.partition_info.batch_index.GetIndex();
@@ -316,6 +316,7 @@ void PhysicalBatchInsert::NextBatch(ExecutionContext &context, GlobalSinkState &
 		lstate.CreateNewCollection(table, insert_types);
 	}
 	lstate.current_index = batch_index;
+	return SinkNextBatchType::READY;
 }
 
 SinkResultType PhysicalBatchInsert::Sink(ExecutionContext &context, DataChunk &chunk, OperatorSinkInput &input) const {
