@@ -12,7 +12,7 @@ struct DuckDBSchemasData : public GlobalTableFunctionState {
 	DuckDBSchemasData() : offset(0) {
 	}
 
-	vector<SchemaCatalogEntry *> entries;
+	vector<reference<SchemaCatalogEntry>> entries;
 	idx_t offset;
 };
 
@@ -40,7 +40,7 @@ static unique_ptr<FunctionData> DuckDBSchemasBind(ClientContext &context, TableF
 }
 
 unique_ptr<GlobalTableFunctionState> DuckDBSchemasInit(ClientContext &context, TableFunctionInitInput &input) {
-	auto result = make_unique<DuckDBSchemasData>();
+	auto result = make_uniq<DuckDBSchemasData>();
 
 	// scan all the schemas and collect them
 	result->entries = Catalog::GetAllSchemas(context);
@@ -49,7 +49,7 @@ unique_ptr<GlobalTableFunctionState> DuckDBSchemasInit(ClientContext &context, T
 }
 
 void DuckDBSchemasFunction(ClientContext &context, TableFunctionInput &data_p, DataChunk &output) {
-	auto &data = (DuckDBSchemasData &)*data_p.global_state;
+	auto &data = data_p.global_state->Cast<DuckDBSchemasData>();
 	if (data.offset >= data.entries.size()) {
 		// finished returning values
 		return;
@@ -58,20 +58,20 @@ void DuckDBSchemasFunction(ClientContext &context, TableFunctionInput &data_p, D
 	// either fill up the chunk or return all the remaining columns
 	idx_t count = 0;
 	while (data.offset < data.entries.size() && count < STANDARD_VECTOR_SIZE) {
-		auto &entry = data.entries[data.offset];
+		auto &entry = data.entries[data.offset].get();
 
 		// return values:
 		idx_t col = 0;
 		// "oid", PhysicalType::BIGINT
-		output.SetValue(col++, count, Value::BIGINT(entry->oid));
+		output.SetValue(col++, count, Value::BIGINT(entry.oid));
 		// database_name, VARCHAR
-		output.SetValue(col++, count, entry->catalog->GetName());
+		output.SetValue(col++, count, entry.catalog.GetName());
 		// database_oid, BIGINT
-		output.SetValue(col++, count, Value::BIGINT(entry->catalog->GetOid()));
+		output.SetValue(col++, count, Value::BIGINT(entry.catalog.GetOid()));
 		// "schema_name", PhysicalType::VARCHAR
-		output.SetValue(col++, count, Value(entry->name));
+		output.SetValue(col++, count, Value(entry.name));
 		// "internal", PhysicalType::BOOLEAN
-		output.SetValue(col++, count, Value::BOOLEAN(entry->internal));
+		output.SetValue(col++, count, Value::BOOLEAN(entry.internal));
 		// "sql", PhysicalType::VARCHAR
 		output.SetValue(col++, count, Value());
 
