@@ -10,7 +10,7 @@ duckdb_data_chunk duckdb_create_data_chunk(duckdb_logical_type *ctypes, idx_t co
 	}
 	duckdb::vector<duckdb::LogicalType> types;
 	for (idx_t i = 0; i < column_count; i++) {
-		auto ltype = (duckdb::LogicalType *)ctypes[i];
+		auto ltype = reinterpret_cast<duckdb::LogicalType *>(ctypes[i]);
 		types.push_back(*ltype);
 	}
 
@@ -21,7 +21,7 @@ duckdb_data_chunk duckdb_create_data_chunk(duckdb_logical_type *ctypes, idx_t co
 
 void duckdb_destroy_data_chunk(duckdb_data_chunk *chunk) {
 	if (chunk && *chunk) {
-		auto dchunk = (duckdb::DataChunk *)*chunk;
+		auto dchunk = reinterpret_cast<duckdb::DataChunk *>(*chunk);
 		delete dchunk;
 		*chunk = nullptr;
 	}
@@ -31,7 +31,7 @@ void duckdb_data_chunk_reset(duckdb_data_chunk chunk) {
 	if (!chunk) {
 		return;
 	}
-	auto dchunk = (duckdb::DataChunk *)chunk;
+	auto dchunk = reinterpret_cast<duckdb::DataChunk *>(chunk);
 	dchunk->Reset();
 }
 
@@ -39,7 +39,7 @@ idx_t duckdb_data_chunk_get_column_count(duckdb_data_chunk chunk) {
 	if (!chunk) {
 		return 0;
 	}
-	auto dchunk = (duckdb::DataChunk *)chunk;
+	auto dchunk = reinterpret_cast<duckdb::DataChunk *>(chunk);
 	return dchunk->ColumnCount();
 }
 
@@ -47,7 +47,7 @@ duckdb_vector duckdb_data_chunk_get_vector(duckdb_data_chunk chunk, idx_t col_id
 	if (!chunk || col_idx >= duckdb_data_chunk_get_column_count(chunk)) {
 		return nullptr;
 	}
-	auto dchunk = (duckdb::DataChunk *)chunk;
+	auto dchunk = reinterpret_cast<duckdb::DataChunk *>(chunk);
 	return reinterpret_cast<duckdb_vector>(&dchunk->data[col_idx]);
 }
 
@@ -55,7 +55,7 @@ idx_t duckdb_data_chunk_get_size(duckdb_data_chunk chunk) {
 	if (!chunk) {
 		return 0;
 	}
-	auto dchunk = (duckdb::DataChunk *)chunk;
+	auto dchunk = reinterpret_cast<duckdb::DataChunk *>(chunk);
 	return dchunk->size();
 }
 
@@ -63,7 +63,7 @@ void duckdb_data_chunk_set_size(duckdb_data_chunk chunk, idx_t size) {
 	if (!chunk) {
 		return;
 	}
-	auto dchunk = (duckdb::DataChunk *)chunk;
+	auto dchunk = reinterpret_cast<duckdb::DataChunk *>(chunk);
 	dchunk->SetCardinality(size);
 }
 
@@ -71,7 +71,7 @@ duckdb_logical_type duckdb_vector_get_column_type(duckdb_vector vector) {
 	if (!vector) {
 		return nullptr;
 	}
-	auto v = (duckdb::Vector *)vector;
+	auto v = reinterpret_cast<duckdb::Vector *>(vector);
 	return reinterpret_cast<duckdb_logical_type>(new duckdb::LogicalType(v->GetType()));
 }
 
@@ -79,7 +79,7 @@ void *duckdb_vector_get_data(duckdb_vector vector) {
 	if (!vector) {
 		return nullptr;
 	}
-	auto v = (duckdb::Vector *)vector;
+	auto v = reinterpret_cast<duckdb::Vector *>(vector);
 	return duckdb::FlatVector::GetData(*v);
 }
 
@@ -87,7 +87,7 @@ uint64_t *duckdb_vector_get_validity(duckdb_vector vector) {
 	if (!vector) {
 		return nullptr;
 	}
-	auto v = (duckdb::Vector *)vector;
+	auto v = reinterpret_cast<duckdb::Vector *>(vector);
 	return duckdb::FlatVector::Validity(*v).GetData();
 }
 
@@ -95,7 +95,7 @@ void duckdb_vector_ensure_validity_writable(duckdb_vector vector) {
 	if (!vector) {
 		return;
 	}
-	auto v = (duckdb::Vector *)vector;
+	auto v = reinterpret_cast<duckdb::Vector *>(vector);
 	auto &validity = duckdb::FlatVector::Validity(*v);
 	validity.EnsureWritable();
 }
@@ -108,7 +108,7 @@ void duckdb_vector_assign_string_element_len(duckdb_vector vector, idx_t index,
 	if (!vector) {
 		return;
 	}
-	auto v = (duckdb::Vector *)vector;
+	auto v = reinterpret_cast<duckdb::Vector *>(vector);
 	auto data = duckdb::FlatVector::GetData<duckdb::string_t>(*v);
 	data[index] = duckdb::StringVector::AddString(*v, str, str_len);
 }
@@ -117,7 +117,7 @@ duckdb_vector duckdb_list_vector_get_child(duckdb_vector vector) {
 	if (!vector) {
 		return nullptr;
 	}
-	auto v = (duckdb::Vector *)vector;
+	auto v = reinterpret_cast<duckdb::Vector *>(vector);
 	return reinterpret_cast<duckdb_vector>(&duckdb::ListVector::GetEntry(*v));
 }
 
@@ -125,7 +125,7 @@ idx_t duckdb_list_vector_get_size(duckdb_vector vector) {
 	if (!vector) {
 		return 0;
 	}
-	auto v = (duckdb::Vector *)vector;
+	auto v = reinterpret_cast<duckdb::Vector *>(vector);
 	return duckdb::ListVector::GetListSize(*v);
 }
 
@@ -133,7 +133,7 @@ duckdb_state duckdb_list_vector_set_size(duckdb_vector vector, idx_t size) {
 	if (!vector) {
 		return duckdb_state::DuckDBError;
 	}
-	auto v = (duckdb::Vector *)vector;
+	auto v = reinterpret_cast<duckdb::Vector *>(vector);
 	duckdb::ListVector::SetListSize(*v, size);
 	return duckdb_state::DuckDBSuccess;
 }
@@ -142,7 +142,7 @@ duckdb_state duckdb_list_vector_reserve(duckdb_vector vector, idx_t required_cap
 	if (!vector) {
 		return duckdb_state::DuckDBError;
 	}
-	auto v = (duckdb::Vector *)vector;
+	auto v = reinterpret_cast<duckdb::Vector *>(vector);
 	duckdb::ListVector::Reserve(*v, required_capacity);
 	return duckdb_state::DuckDBSuccess;
 }
@@ -151,7 +151,7 @@ duckdb_vector duckdb_struct_vector_get_child(duckdb_vector vector, idx_t index)
 	if (!vector) {
 		return nullptr;
 	}
-	auto v = (duckdb::Vector *)vector;
+	auto v = reinterpret_cast<duckdb::Vector *>(vector);
 	return reinterpret_cast<duckdb_vector>(duckdb::StructVector::GetEntries(*v)[index].get());
 }
 
