@@ -106,6 +106,58 @@ bool PipelineExecutor::TryFlushCachingOperators() {
 	return true;
 }
 
+SinkNextBatchType PipelineExecutor::NextBatch(duckdb::DataChunk &source_chunk) {
+	D_ASSERT(requires_batch_index);
+	idx_t next_batch_index;
+	if (source_chunk.size() == 0) {
+		next_batch_index = NumericLimits<int64_t>::Maximum();
+	} else {
+		next_batch_index =
+		    pipeline.source->GetBatchIndex(context, source_chunk, *pipeline.source_state, *local_source_state);
+		// we start with the base_batch_index as a valid starting value. Make sure that next batch is called below
+		next_batch_index += pipeline.base_batch_index + 1;
+	}
+	auto &partition_info = local_sink_state->partition_info;
+	if (next_batch_index == partition_info.batch_index.GetIndex()) {
+		// no changes, return
+		return SinkNextBatchType::READY;
+	}
+	// batch index has changed - update it
+	if (partition_info.batch_index.GetIndex() > next_batch_index) {
+		throw InternalException(
+		    "Pipeline batch index - gotten lower batch index %llu (down from previous batch index of %llu)",
+		    next_batch_index, partition_info.batch_index.GetIndex());
+	}
+#ifdef DUCKDB_DEBUG_ASYNC_SINK_SOURCE
+	if (debug_blocked_next_batch_count < debug_blocked_target_count) {
+		debug_blocked_next_batch_count++;
+
+		auto &callback_state = interrupt_state;
+		std::thread rewake_thread([callback_state] {
+			std::this_thread::sleep_for(std::chrono::milliseconds(1));
+			callback_state.Callback();
+		});
+		rewake_thread.detach();
+
+		return SinkNextBatchType::BLOCKED;
+	}
+#endif
+	auto current_batch = partition_info.batch_index.GetIndex();
+	partition_info.batch_index = next_batch_index;
+	OperatorSinkNextBatchInput next_batch_input {*pipeline.sink->sink_state, *local_sink_state, interrupt_state};
+	// call NextBatch before updating min_batch_index to provide the opportunity to flush the previous batch
+	auto next_batch_result = pipeline.sink->NextBatch(context, next_batch_input);
+
+	if (next_batch_result == SinkNextBatchType::BLOCKED) {
+		partition_info.batch_index = current_batch; // set batch_index back to what it was before
+		return SinkNextBatchType::BLOCKED;
+	}
+
+	partition_info.min_batch_index = pipeline.UpdateBatchIndex(current_batch, next_batch_index);
+
+	return SinkNextBatchType::READY;
+}
+
 PipelineExecuteResult PipelineExecutor::Execute(idx_t max_chunks) {
 	D_ASSERT(pipeline.sink);
 	auto &source_chunk = pipeline.operators.empty() ? final_chunk : *intermediate_chunks[0];
@@ -115,7 +167,8 @@ PipelineExecuteResult PipelineExecutor::Execute(idx_t max_chunks) {
 		}
 
 		OperatorResultType result;
-		if (exhausted_source && done_flushing && !remaining_sink_chunk && in_process_operators.empty()) {
+		if (exhausted_source && done_flushing && !remaining_sink_chunk && !next_batch_blocked &&
+		    in_process_operators.empty()) {
 			break;
 		} else if (remaining_sink_chunk) {
 			// The pipeline was interrupted by the Sink. We should retry sinking the final chunk.
@@ -127,7 +180,7 @@ PipelineExecuteResult PipelineExecutor::Execute(idx_t max_chunks) {
 			// the result for the pipeline
 			D_ASSERT(source_chunk.size() > 0);
 			result = ExecutePushInternal(source_chunk);
-		} else if (exhausted_source && !done_flushing) {
+		} else if (exhausted_source && !next_batch_blocked && !done_flushing) {
 			// The source was exhausted, try flushing all operators
 			auto flush_completed = TryFlushCachingOperators();
 			if (flush_completed) {
@@ -136,21 +189,33 @@ PipelineExecuteResult PipelineExecutor::Execute(idx_t max_chunks) {
 			} else {
 				return PipelineExecuteResult::INTERRUPTED;
 			}
-		} else if (!exhausted_source) {
-			// "Regular" path: fetch a chunk from the source and push it through the pipeline
-			source_chunk.Reset();
-			SourceResultType source_result = FetchFromSource(source_chunk);
-
-			if (source_result == SourceResultType::BLOCKED) {
-				return PipelineExecuteResult::INTERRUPTED;
-			}
-
-			if (source_result == SourceResultType::FINISHED) {
-				exhausted_source = true;
-				if (source_chunk.size() == 0) {
-					continue;
+		} else if (!exhausted_source || next_batch_blocked) {
+			SourceResultType source_result;
+			if (!next_batch_blocked) {
+				// "Regular" path: fetch a chunk from the source and push it through the pipeline
+				source_chunk.Reset();
+				source_result = FetchFromSource(source_chunk);
+				if (source_result == SourceResultType::BLOCKED) {
+					return PipelineExecuteResult::INTERRUPTED;
+				}
+				if (source_result == SourceResultType::FINISHED) {
+					exhausted_source = true;
 				}
 			}
+
+			if (requires_batch_index) {
+				auto next_batch_result = NextBatch(source_chunk);
+				next_batch_blocked = next_batch_result == SinkNextBatchType::BLOCKED;
+				if (next_batch_blocked) {
+					return PipelineExecuteResult::INTERRUPTED;
+				}
+			}
+
+			if (exhausted_source && source_chunk.size() == 0) {
+				// To ensure that we're not early-terminating the pipeline
+				continue;
+			}
+
 			result = ExecutePushInternal(source_chunk);
 		} else {
 			throw InternalException("Unexpected state reached in pipeline executor");
@@ -285,8 +350,9 @@ void PipelineExecutor::ExecutePull(DataChunk &result) {
 	auto &executor = pipeline.executor;
 	try {
 		D_ASSERT(!pipeline.sink);
+		D_ASSERT(!requires_batch_index);
 		auto &source_chunk = pipeline.operators.empty() ? result : *intermediate_chunks[0];
-		while (result.size() == 0 && !exhausted_source) {
+		while (result.size() == 0 && (!exhausted_source || !in_process_operators.empty())) {
 			if (in_process_operators.empty()) {
 				source_chunk.Reset();
 
@@ -296,6 +362,7 @@ void PipelineExecutor::ExecutePull(DataChunk &result) {
 
 				// Repeatedly try to fetch from the source until it doesn't block. Note that it may block multiple times
 				while (true) {
+					D_ASSERT(!exhausted_source);
 					source_result = FetchFromSource(source_chunk);
 
 					// No interrupt happened, all good.
@@ -489,32 +556,6 @@ SourceResultType PipelineExecutor::FetchFromSource(DataChunk &result) {
 	// Ensures Sinks only return empty results when Blocking or Finished
 	D_ASSERT(res != SourceResultType::BLOCKED || result.size() == 0);
 
-	if (requires_batch_index && res != SourceResultType::BLOCKED) {
-		idx_t next_batch_index;
-		if (result.size() == 0) {
-			next_batch_index = NumericLimits<int64_t>::Maximum();
-		} else {
-			next_batch_index =
-			    pipeline.source->GetBatchIndex(context, result, *pipeline.source_state, *local_source_state);
-			// we start with the base_batch_index as a valid starting value. Make sure that next batch is called below
-			next_batch_index += pipeline.base_batch_index + 1;
-		}
-		auto &partition_info = local_sink_state->partition_info;
-		if (next_batch_index != partition_info.batch_index.GetIndex()) {
-			// batch index has changed - update it
-			if (partition_info.batch_index.GetIndex() > next_batch_index) {
-				throw InternalException(
-				    "Pipeline batch index - gotten lower batch index %llu (down from previous batch index of %llu)",
-				    next_batch_index, partition_info.batch_index.GetIndex());
-			}
-			auto current_batch = partition_info.batch_index.GetIndex();
-			partition_info.batch_index = next_batch_index;
-			// call NextBatch before updating min_batch_index to provide the opportunity to flush the previous batch
-			pipeline.sink->NextBatch(context, *pipeline.sink->sink_state, *local_sink_state);
-			partition_info.min_batch_index = pipeline.UpdateBatchIndex(current_batch, next_batch_index);
-		}
-	}
-
 	EndOperator(*pipeline.source, &result);
 
 	return res;
