@@ -3,11 +3,11 @@
 namespace duckdb {
 
 TransactionStatement::TransactionStatement(TransactionType type)
-    : SQLStatement(StatementType::TRANSACTION_STATEMENT), info(make_unique<TransactionInfo>(type)) {
+    : SQLStatement(StatementType::TRANSACTION_STATEMENT), info(make_uniq<TransactionInfo>(type)) {
 }
 
 TransactionStatement::TransactionStatement(const TransactionStatement &other)
-    : SQLStatement(other), info(make_unique<TransactionInfo>(other.info->type)) {
+    : SQLStatement(other), info(make_uniq<TransactionInfo>(other.info->type)) {
 }
 
 unique_ptr<SQLStatement> TransactionStatement::Copy() const {
