@@ -4,12 +4,12 @@
 
 namespace duckdb {
 
-unique_ptr<ParsedExpression> Transformer::TransformPositionalReference(duckdb_libpgquery::PGPositionalReference *node) {
-	if (node->position <= 0) {
+unique_ptr<ParsedExpression> Transformer::TransformPositionalReference(duckdb_libpgquery::PGPositionalReference &node) {
+	if (node.position <= 0) {
 		throw ParserException("Positional reference node needs to be >= 1");
 	}
-	auto result = make_uniq<PositionalReferenceExpression>(node->position);
-	result->query_location = node->location;
+	auto result = make_uniq<PositionalReferenceExpression>(node.position);
+	result->query_location = node.location;
 	return std::move(result);
 }
 
