@@ -3,7 +3,6 @@
 
 #include "duckdb/catalog/catalog_entry/aggregate_function_catalog_entry.hpp"
 #include "duckdb/common/types/hash.hpp"
-#include "duckdb/common/field_writer.hpp"
 #include "duckdb/planner/expression/bound_cast_expression.hpp"
 #include "duckdb/function/function_serialization.hpp"
 
@@ -15,7 +14,7 @@ BoundAggregateExpression::BoundAggregateExpression(AggregateFunction function, v
     : Expression(ExpressionType::BOUND_AGGREGATE, ExpressionClass::BOUND_AGGREGATE, function.return_type),
       function(std::move(function)), children(std::move(children)), bind_info(std::move(bind_info)),
       aggr_type(aggr_type), filter(std::move(filter)) {
-	D_ASSERT(!function.name.empty());
+	D_ASSERT(!this->function.name.empty());
 }
 
 string BoundAggregateExpression::ToString() const {
@@ -81,27 +80,27 @@ unique_ptr<Expression> BoundAggregateExpression::Copy() {
 	return std::move(copy);
 }
 
-void BoundAggregateExpression::Serialize(FieldWriter &writer) const {
-	writer.WriteField(IsDistinct());
-	writer.WriteOptional(filter);
-	writer.WriteOptional(order_bys);
-	FunctionSerializer::Serialize<AggregateFunction>(writer, function, return_type, children, bind_info.get());
+void BoundAggregateExpression::Serialize(Serializer &serializer) const {
+	Expression::Serialize(serializer);
+	serializer.WriteProperty(200, "return_type", return_type);
+	serializer.WriteProperty(201, "children", children);
+	FunctionSerializer::Serialize(serializer, function, bind_info.get());
+	serializer.WriteProperty(203, "aggregate_type", aggr_type);
+	serializer.WritePropertyWithDefault(204, "filter", filter, unique_ptr<Expression>());
+	serializer.WritePropertyWithDefault(205, "order_bys", order_bys, unique_ptr<BoundOrderModifier>());
 }
 
-unique_ptr<Expression> BoundAggregateExpression::Deserialize(ExpressionDeserializationState &state,
-                                                             FieldReader &reader) {
-	auto distinct = reader.ReadRequired<bool>();
-	auto filter = reader.ReadOptional<Expression>(nullptr, state.gstate);
-	auto order_bys = reader.ReadOptional<BoundOrderModifier>(nullptr, state.gstate);
-	vector<unique_ptr<Expression>> children;
-	unique_ptr<FunctionData> bind_info;
-	auto function = FunctionSerializer::Deserialize<AggregateFunction, AggregateFunctionCatalogEntry>(
-	    reader, state, CatalogType::AGGREGATE_FUNCTION_ENTRY, children, bind_info);
-
-	auto x = make_uniq<BoundAggregateExpression>(function, std::move(children), std::move(filter), std::move(bind_info),
-	                                             distinct ? AggregateType::DISTINCT : AggregateType::NON_DISTINCT);
-	x->order_bys = std::move(order_bys);
-	return std::move(x);
+unique_ptr<Expression> BoundAggregateExpression::Deserialize(Deserializer &deserializer) {
+	auto return_type = deserializer.ReadProperty<LogicalType>(200, "return_type");
+	auto children = deserializer.ReadProperty<vector<unique_ptr<Expression>>>(201, "children");
+	auto entry = FunctionSerializer::Deserialize<AggregateFunction, AggregateFunctionCatalogEntry>(
+	    deserializer, CatalogType::AGGREGATE_FUNCTION_ENTRY, children, std::move(return_type));
+	auto aggregate_type = deserializer.ReadProperty<AggregateType>(203, "aggregate_type");
+	auto filter = deserializer.ReadPropertyWithDefault<unique_ptr<Expression>>(204, "filter", unique_ptr<Expression>());
+	auto result = make_uniq<BoundAggregateExpression>(std::move(entry.first), std::move(children), std::move(filter),
+	                                                  std::move(entry.second), aggregate_type);
+	deserializer.ReadPropertyWithDefault(205, "order_bys", result->order_bys, unique_ptr<BoundOrderModifier>());
+	return std::move(result);
 }
 
 } // namespace duckdb
