@@ -4,7 +4,6 @@
 #include "duckdb/planner/expression/bound_columnref_expression.hpp"
 #include "duckdb/planner/expression/bound_reference_expression.hpp"
 #include "duckdb/storage/table/append_state.hpp"
-#include "duckdb/execution/index/art/art.hpp"
 
 namespace duckdb {
 
@@ -13,7 +12,7 @@ Index::Index(AttachedDatabase &db, IndexType type, TableIOManager &table_io_mana
              IndexConstraintType constraint_type_p)
 
     : type(type), table_io_manager(table_io_manager), column_ids(column_ids_p), constraint_type(constraint_type_p),
-      db(db), buffer_manager(BufferManager::GetBufferManager(db)) {
+      db(db) {
 
 	for (auto &expr : unbound_expressions) {
 		types.push_back(expr->return_type.InternalType());
@@ -40,6 +39,12 @@ PreservedError Index::Append(DataChunk &entries, Vector &row_identifiers) {
 	return Append(state, entries, row_identifiers);
 }
 
+void Index::CommitDrop() {
+	IndexLock index_lock;
+	InitializeLock(index_lock);
+	CommitDrop(index_lock);
+}
+
 void Index::Delete(DataChunk &entries, Vector &row_identifiers) {
 	IndexLock state;
 	InitializeLock(state);
@@ -47,42 +52,21 @@ void Index::Delete(DataChunk &entries, Vector &row_identifiers) {
 }
 
 bool Index::MergeIndexes(Index &other_index) {
-
 	IndexLock state;
 	InitializeLock(state);
-
-	switch (this->type) {
-	case IndexType::ART:
-		return Cast<ART>().MergeIndexes(state, other_index);
-	default:
-		throw InternalException("Unimplemented index type for merge");
-	}
+	return MergeIndexes(state, other_index);
 }
 
 string Index::VerifyAndToString(const bool only_verify) {
-
 	IndexLock state;
 	InitializeLock(state);
-
-	switch (this->type) {
-	case IndexType::ART:
-		return Cast<ART>().VerifyAndToString(state, only_verify);
-	default:
-		throw InternalException("Unimplemented index type for VerifyAndToString");
-	}
+	return VerifyAndToString(state, only_verify);
 }
 
 void Index::Vacuum() {
-
 	IndexLock state;
 	InitializeLock(state);
-
-	switch (this->type) {
-	case IndexType::ART:
-		return Cast<ART>().Vacuum(state);
-	default:
-		throw InternalException("Unimplemented index type for vacuum");
-	}
+	Vacuum(state);
 }
 
 void Index::ExecuteExpressions(DataChunk &input, DataChunk &result) {
@@ -108,7 +92,7 @@ bool Index::IndexIsUpdated(const vector<PhysicalIndex> &column_ids) const {
 	return false;
 }
 
-BlockPointer Index::Serialize(MetaBlockWriter &writer) {
+BlockPointer Index::Serialize(MetadataWriter &writer) {
 	throw NotImplementedException("The implementation of this index serialization does not exist.");
 }
 
