@@ -1,10 +1,13 @@
 #include "duckdb/storage/statistics/string_stats.hpp"
-#include "duckdb/storage/statistics/base_statistics.hpp"
-#include "duckdb/common/field_writer.hpp"
-#include "utf8proc_wrapper.hpp"
+
 #include "duckdb/common/string_util.hpp"
 #include "duckdb/common/types/vector.hpp"
 #include "duckdb/main/error_manager.hpp"
+#include "duckdb/storage/statistics/base_statistics.hpp"
+#include "utf8proc_wrapper.hpp"
+
+#include "duckdb/common/serializer/serializer.hpp"
+#include "duckdb/common/serializer/deserializer.hpp"
 
 namespace duckdb {
 
@@ -67,6 +70,24 @@ bool StringStats::CanContainUnicode(const BaseStatistics &stats) {
 	return StringStats::GetDataUnsafe(stats).has_unicode;
 }
 
+string GetStringMinMaxValue(const data_t data[]) {
+	idx_t len;
+	for (len = 0; len < StringStatsData::MAX_STRING_MINMAX_SIZE; len++) {
+		if (!data[len]) {
+			break;
+		}
+	}
+	return string(const_char_ptr_cast(data), len);
+}
+
+string StringStats::Min(const BaseStatistics &stats) {
+	return GetStringMinMaxValue(StringStats::GetDataUnsafe(stats).min);
+}
+
+string StringStats::Max(const BaseStatistics &stats) {
+	return GetStringMinMaxValue(StringStats::GetDataUnsafe(stats).max);
+}
+
 void StringStats::ResetMaxStringLength(BaseStatistics &stats) {
 	StringStats::GetDataUnsafe(stats).has_max_string_length = false;
 }
@@ -75,24 +96,22 @@ void StringStats::SetContainsUnicode(BaseStatistics &stats) {
 	StringStats::GetDataUnsafe(stats).has_unicode = true;
 }
 
-void StringStats::Serialize(const BaseStatistics &stats, FieldWriter &writer) {
+void StringStats::Serialize(const BaseStatistics &stats, Serializer &serializer) {
 	auto &string_data = StringStats::GetDataUnsafe(stats);
-	writer.WriteBlob(string_data.min, StringStatsData::MAX_STRING_MINMAX_SIZE);
-	writer.WriteBlob(string_data.max, StringStatsData::MAX_STRING_MINMAX_SIZE);
-	writer.WriteField<bool>(string_data.has_unicode);
-	writer.WriteField<bool>(string_data.has_max_string_length);
-	writer.WriteField<uint32_t>(string_data.max_string_length);
+	serializer.WriteProperty(200, "min", string_data.min, StringStatsData::MAX_STRING_MINMAX_SIZE);
+	serializer.WriteProperty(201, "max", string_data.max, StringStatsData::MAX_STRING_MINMAX_SIZE);
+	serializer.WriteProperty(202, "has_unicode", string_data.has_unicode);
+	serializer.WriteProperty(203, "has_max_string_length", string_data.has_max_string_length);
+	serializer.WriteProperty(204, "max_string_length", string_data.max_string_length);
 }
 
-BaseStatistics StringStats::Deserialize(FieldReader &reader, LogicalType type) {
-	BaseStatistics result(std::move(type));
-	auto &string_data = StringStats::GetDataUnsafe(result);
-	reader.ReadBlob(string_data.min, StringStatsData::MAX_STRING_MINMAX_SIZE);
-	reader.ReadBlob(string_data.max, StringStatsData::MAX_STRING_MINMAX_SIZE);
-	string_data.has_unicode = reader.ReadRequired<bool>();
-	string_data.has_max_string_length = reader.ReadRequired<bool>();
-	string_data.max_string_length = reader.ReadRequired<uint32_t>();
-	return result;
+void StringStats::Deserialize(Deserializer &deserializer, BaseStatistics &base) {
+	auto &string_data = StringStats::GetDataUnsafe(base);
+	deserializer.ReadProperty(200, "min", string_data.min, StringStatsData::MAX_STRING_MINMAX_SIZE);
+	deserializer.ReadProperty(201, "max", string_data.max, StringStatsData::MAX_STRING_MINMAX_SIZE);
+	deserializer.ReadProperty(202, "has_unicode", string_data.has_unicode);
+	deserializer.ReadProperty(203, "has_max_string_length", string_data.has_max_string_length);
+	deserializer.ReadProperty(204, "max_string_length", string_data.max_string_length);
 }
 
 static int StringValueComparison(const_data_ptr_t data, idx_t len, const_data_ptr_t comparison) {
