@@ -34,10 +34,14 @@
 
 #include "src/core_functions/scalar/string/reverse.cpp"
 
+#include "src/core_functions/scalar/string/sha256.cpp"
+
 #include "src/core_functions/scalar/string/starts_with.cpp"
 
 #include "src/core_functions/scalar/string/string_split.cpp"
 
+#include "src/core_functions/scalar/string/to_base.cpp"
+
 #include "src/core_functions/scalar/string/translate.cpp"
 
 #include "src/core_functions/scalar/string/trim.cpp"
